<?php
/**
 * REST API
 */

namespace SangoBlocks;

class CustomJS {
  private $codes = [];

  public function init() {
    add_action('wp_footer', [$this, 'print_codes'], 9999);
  }

  public function print_codes() {
    if (empty($this->codes)) {
      return;
    }
    $final_js = $this->ready_js();
    $final_js = $this->minify_js($final_js);
    $codes = $this->codes;
    foreach ($codes as $code) {
      $final_js .= $code;
    }
    echo '<script>' . $final_js . '</script>';
  }

  public function register($key, $code) {
    $script = <<<EOT
    sgb.domReady(function() {
      const block = document.querySelector("#$key");
      $code
    });
EOT;
    $this->codes[$key] = $script;
  }

  public function ready_js() {
    $script = <<<EOT
    const sgb = {};
    sgb.domReady = (fn) => {
      document.addEventListener("DOMContentLoaded", fn);
      if (document.readyState === "interactive" || document.readyState === "complete" ) {
        fn();
      }
    };
EOT;
    return $script;
  }

  public function minify_js($js) {
    return preg_replace(
      array(
        // Remove comments
        '#\s*("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')\s*|\s*\/\*(?!\!|@cc_on)(?>[\s\S]*?\*\/)\s*|\s*(?<![\:\=])\/\/.*(?=[\n\r]|$)|^\s*|\s*$#',
        // Remove white-spaces
        '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\'|\/\*(?>.*?\*\/)|\/(?!\/)[^\n\r]*?\/(?=[\s.,;]|[gimuy]|$))|\s*([!%&*\(\)\-=+\[\]\{\}|;:,.<>?\/])\s*#s',
        // Remove the last semicolon
        '#;+\}#',
      ),
      array(
        '$1',
        '$1$2',
        '}'
      ),
    $js);
  }
}
